package com.yaoxiang.planning.service

import com.yaoxiang.planning.config.PlanningProperties
import com.yaoxiang.planning.domain.Department
import com.yaoxiang.planning.repository.DepartmentRepo
import com.yaoxiang.planning.utils.TreeUtil
import mu.KotlinLogging
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.util.*
import javax.annotation.PostConstruct

private val logger = KotlinLogging.logger { }

@Service
class DepartmentService {

    @Autowired
    private lateinit var departmentRepo: DepartmentRepo

    @Autowired
    private lateinit var userService: UserService

    @Autowired
    private lateinit var planningProperties: PlanningProperties

    @PostConstruct
    fun init() {
        val list = list()
        if (list.isEmpty()) {
            add(planningProperties.departmentName, null, null, null)
            logger.info { "init root department." }
        }
    }

    fun list(): List<Department> {
        return departmentRepo.findAll()
    }

    fun listAllChildren(rootId: Long): List<Department> {
        val optional = get(rootId)
        if (!optional.isPresent) {
            return arrayListOf()
        }
        val list = list()
//        val map = list.associateBy { it.id!! }
        TreeUtil.listAllChildren(optional.get(), list.toMutableList())
        return list
    }

    @Transactional
    fun add(name: String, duty: String?, remark: String?, parentId: Long?): Boolean {
        if (existsByName(name)) {
            return false
        }
        if (parentId == null && findRoot().isPresent) {
            return false
        }
        if (parentId != null && !exists(parentId)) {
            return false
        }
        val department = Department()
        department.name = name
        department.duty = duty
        department.remark = remark
        department.parentId = parentId
        departmentRepo.save(department)
        logger.info { "添加部门成功,name=${name},duty=${duty},remark=${remark},parentId=${parentId}" }
        return true
    }

    @Transactional
    fun update(id: Long, name: String, duty: String?, remark: String?, parentId: Long?): Boolean {
        val optional = get(id)
        if (!optional.isPresent) {
            return false
        }
        val department = optional.get()
        if (parentId == null && department.parentId != null) {
            logger.error { "不允许设置非root部门的上级部门为null,departmentName==${department.name}" }
            return false
        }
        if (parentId != null && department.parentId == null) {
            logger.error { "不允许设置root部门的parentId,departmentName=${department.name},parentId=${parentId}" }
            return false
        }

        department.name = name
        department.duty = duty
        department.remark = remark
        department.parentId = parentId
        departmentRepo.save(department)
        logger.info { "更新部门成功,department=${department}" }
        return true
    }

    @Transactional
    fun delete(id: Long): Boolean {
        val optional = get(id)
        if (!optional.isPresent) {
            return false
        }
        val department = optional.get()
        if (department.parentId == null) {
            logger.error { "不允许删除root部门,id=${id}" }
            return false
        }
        val list = list()
        val children = TreeUtil.listChildren(department, list.toMutableList()) as List<Department>
        departmentRepo.deleteAll(children)
        userService.clearDepartment(children.map { it.id!! })
        logger.info { "删除部门成功,id=${id},departmentName=${department.name},size=${children.size}" }
        return true
    }

    fun exists(id: Long): Boolean {
        return departmentRepo.existsById(id)
    }

    private fun existsByName(name: String): Boolean {
        return departmentRepo.existsByName(name)
    }

    fun get(id: Long): Optional<Department> {
        return departmentRepo.findById(id)
    }

    fun findRoot(): Optional<Department> {
        return departmentRepo.findFirstByParentIdIsNull()
    }

    @Transactional(readOnly = true)
    fun getTree(id: Long, withUser: Boolean): Optional<Department> {
        val list = listAllChildren(id)
        if (list.isEmpty()) {
            return Optional.empty()
        }
        val root = list.firstOrNull { id == it.id!! }
        if (withUser) {
            val users = userService.list()
            val map = users.groupBy { it.departmentId }
            list.forEach { it.userList = map.getOrDefault(it.id, arrayListOf()) }
            //只给root部门加上userList
            if (root != null && root.parentId == null && root.userList == null) {
                root.userList = map[null]
            }
        }
//        val map = list.associateBy { it.id!! }
        return Optional.ofNullable(root)
    }

    fun getName(id: Long?): String? {
        if (id != null) {
            val optional = get(id)
            if (optional.isPresent) {
                return optional.get().name
            }
        }
        return null
    }

}